# Check that mirrors are functioning.

if (($OPT_PHASE_A)); then

    if (($OPT_BACKPORTS)); then
        inform "checking that both the regular mirror"\
        "you have specified and the Debian's Project's snapshot service"\
        "are available and functioning..."
    else
        inform "checking that the regular mirror"\
        "you have specified is available and functioning..."
    fi
    function bail_from_phase_A {
        die "$1 does not properly respond"
    }
    function bail_from_phase_A_for_primary {
        bail_from_phase_A "mirror $MIRROR_PRIMARY"
    }
    function bail_from_phase_A_for_cd {
        bail_from_phase_A "iso/cd mirror $MIRROR_PRIMARY"
    }
    function bail_from_phase_A_for_snapshot {
        bail_from_phase_A "snapshot server $SNAPSHOT_SERVICE"
    }
    mkdir $VV $TMPDIR/mirror-checks{-incoming,}

    rsync -tv --progress --stats\
     "$MIRROR_PRIMARY::$ARCHIVE_NAME/$MIRROR_TEST_INCOMING"\
     "$TMPDIR/mirror-checks-incoming"\
     || bail_from_phase_A_for_primary
    mv $VV "$TMPDIR/mirror-checks-incoming/$MIRROR_TEST_INCOMING"\
     "$TMPDIR/mirror-checks/$MIRROR_TEST"\
     || bail_from_phase_A_for_primary
    [ -s "$TMPDIR/mirror-checks/$MIRROR_TEST" ]\
     || bail_from_phase_A_for_primary

    rsync -tv --progress --stats\
     "$MIRROR_PRIMARY::$CD_ARCHIVE_NAME/$CD_MIRROR_TEST_INCOMING"\
     "$TMPDIR/mirror-checks-incoming"\
     || bail_from_phase_A_for_cd
    mv $VV "$TMPDIR/mirror-checks-incoming/$CD_MIRROR_TEST_INCOMING"\
     "$TMPDIR/mirror-checks/$CD_MIRROR_TEST"\
     || bail_from_phase_A_for_cd
    [ -s "$TMPDIR/mirror-checks/$CD_MIRROR_TEST" ]\
     || bail_from_phase_A_for_cd

    (($OPT_BACKPORTS)) && {
        wget >/dev/null 2>/dev/null -nH -O- -P\
         "$TMPDIR/mirror-checks-incoming"\
         "https://${SNAPSHOT_SERVICE%%/*}/static/auwobymhnpx01.html"\
         || true
        # The next command helps administrators of snapshot.debian.org [i] to
        # measure the (presumably modest) fraction for which mirrorrib
        # installations like yours are collectively responsible of the overall
        # load on the service, [ii] to understand the reason for what might
        # otherwise appear to be mysterious, odd, or suspicious accesses by
        # you, and [iii] to advise this script's maintainer to fix any
        # unexpected problems the script might inadvertently be causing the
        # service. The file the command attempts to fetch does not exist. All
        # mirrorrib installations use the same filename (for it is not the
        # filename but your IP address that distinguishes you from other
        # mirrorrib users). If modifying this script, you may strike
        # the "2>/dev/null" from the command if you prefer mirrorrib's standard
        # error stream to report the command's intentional failure but,
        # otherwise, unless you have reconfigured your mirrorrib installation
        # not to access snapshot.debian.org at all, please avoid altering
        # the command.
        wget -nH -P "$TMPDIR/mirror-checks-incoming"\
         "https://$SNAPSHOT_SERVICE/$ARCHIVE_NAME/"\
         || bail_from_phase_A_for_snapshot
        mv $VV "$TMPDIR/mirror-checks-incoming/index.html"\
         "$TMPDIR/mirror-checks/$SNAPSHOT_TOC1"\
         || bail_from_phase_A_for_snapshot
        [ -s "$TMPDIR/mirror-checks/$SNAPSHOT_TOC1" ]\
         || bail_from_phase_A_for_snapshot
    }

    if (($OPT_BACKPORTS)); then
        inform "check complete: mirror and snapshot both respond"
    else
        inform "check complete: mirror responds"
    fi

fi

# The file $TMPDIR/mirror-checks/$SNAPSHOT_TOC1, used during this phase A to
# verify snapshot's responsiveness, will find another use during phase B.
# Phase B need not fetch it again.

