# This file implements two phases and an auxiliary phase.
# Phase  C: fetch dist-root control.
# Phase  D: fetch per-area control.
# Phase DD: stamp the release time.

(($OPT_PHASE_C)) && inform "fetching control files"\
 "to learn what to mirror for Debian $VER $DIST..."

if (($OPT_PHASE_C)); then

    # Predelete the target hierarchy.
    #rm -rf$V "$TARGET" # clumsy
    mkdir -p$V "$TARGET"

fi # phase C continues below, though

for DIST1 in "$DIST" $( (($OPT_BACKPORTS)) && echo "$DIST_BACKPORTS" ); do

    # Appraise whether backports.
    IS_BACKPORTS=1; [ "$DIST1" = "$DIST" ] && IS_BACKPORTS=0
    MIRROR="$MIRROR_PRIMARY"; (($IS_BACKPORTS)) && MIRROR="$SNAPSHOT_SERVICE"

    (($IS_BACKPORTS && $OPT_PHASE_B)) && execute_phase_B

    # -------------------------------------------------------------------------
    if (($OPT_PHASE_C)); then

        while true; do
            # Fetch dist-root control. (Note: in dist-root-ctrl.list, a
            # leading '!' indicates that backports lacks the named control.)
            ROOTFPS=''
            SED_SCRIPT='s/^!//;p'; (($IS_BACKPORTS)) && SED_SCRIPT='/^!/!p'
            for FP1 in $(
                sed -rne "$SED_SCRIPT"\
                "$LIST_DIR/dist-root-ctrl.list"
            ); do
                ROOTFPS="$ROOTFPS dists/$DIST1/$FP1"
            done
            recursively_fetch $ROOTFPS
            (($IS_BACKPORTS)) && break
            # For the stable archive only, fetch dist-root control again, just
            # in case a new Debian release happens to have arrived during the
            # last fetch.
            if [ -e $TMPDIR/zzz-duplicate ]; then
                # Ensure that the two dist-root controls are identical.
                diff -rNq $TARGET/zzz-dists $TMPDIR/zzz-duplicate && break
                pause_before_trying
                rm -rf$V $TMPDIR/zzz-duplicate
            fi
            mv $VV $TARGET/zzz-dists $TMPDIR/zzz-duplicate
        done

    fi # end phase C
    # -------------------------------------------------------------------------
    if (($OPT_PHASE_D)); then

        # Fetch the dist primary control (as introduced by dist-root ctrl).
        # Also, incidentally, list files for later decompression.
        RFPS=''
        ZFPS=''
        (($OPT_SUMS)) && touch $TMPDIR/ctrl-filepaths.list
        ((!$OPT_SUMS)) && inform "scanning ${DIST1}'s Release file..."
        while read SHA256 SIZE FILEPATH; do
            DIR="$(dirname "$FILEPATH")"
            BASE="$(basename "$FILEPATH")"
            if is_ctrl_wanted per-area-ctrl.list "$FILEPATH"; then
                RFPS="$RFPS dists/$DIST1/$FILEPATH"
                if (($OPT_SUMS)); then
                    echo "$SHA256  dists/$DIST1/$FILEPATH"\
                    >>$TMPDIR/ctrl-filepaths.list
                else
                    is_ctrl_wanted ctrl-to-decompress.list "$FILEPATH"\
                    && ZFPS="$ZFPS dists/$DIST1/$FILEPATH"
                fi
            fi
        done < <(
            sed -rne '/^SHA256:\s*$/!d;:b;n;/^ /!q;p;bb'\
            "$TARGET/dists/$DIST1/Release"
        )
        if ((!$OPT_SUMS)); then

            recursively_fetch $RFPS $(
                ((!IS_BACKPORTS)) && echo\
                "dists/$DIST/main/installer-$ARCH/current"
            )
            PREFETCH_FPS=''
            for FP1 in $(eval echo $(<"$LIST_DIR/nodes-to-prefetch.list")); do
                [[ "$FP1" = dists/$(printf '%q' "$DIST1")?(/*) ]]\
                && PREFETCH_FPS="$PREFETCH_FPS $FP1"
            done
            [ -n "$PREFETCH_FPS" ] && recursively_fetch $PREFETCH_FPS

            # Decompress.
            decompress $ZFPS

        fi

    fi # end phase D
    # -------------------------------------------------------------------------
    if (($OPT_PHASE_DD)); then
        # Stamp the release time. (The following if-then-else-fi depends on
        # the enclosing loop to treat backports last.)
        if (($IS_BACKPORTS)); then
            stamp_per_release "$TMPDIR/backports.stamp"\
             "$TARGET/dists/$DIST1/Release"
            [\
                     "$TMPDIR/backports.stamp" -ot "$TMPDIR/now.stamp"\
                -a ! "$TMPDIR/backports.stamp" -nt "$TMPDIR/release.stamp"\
            ] || {
                die "the backports release must lie"\
                "in the past according to the system's clock and, also,"\
                "must precede or equal the stable release"
            }
            ((!$OPT_PHASE_B)) || ! [\
                   "$TMPDIR/release.stamp" -ot "$TMPDIR/phb-release.stamp"\
                -o "$TMPDIR/release.stamp" -nt "$TMPDIR/phb-release.stamp"\
            ] || {
                die "an internal glitch regarding"\
                "timestamps has arisen regarding"\
                "the stable release; the program's"\
                "internal phases B and D conflict (this should not happen"\
                "and, therefore, you should not be reading this message;"\
                "it probably means either that an external agent has disturbed"\
                "program's temporary runtime data files or that there is"\
                "a bug in the program; however, it is barely conceivable that"\
                "the glitch is instead due to unexpected behavior by the mirror)"
            }
        else
            stamp_per_release "$TMPDIR/release.stamp"\
             "$TARGET/dists/$DIST1/Release"
            [ "$TMPDIR/release.stamp" -ot "$TMPDIR/now.stamp" ] || {
                die "the stable release must lie"\
                "in the past according to the system's clock"
            }
        fi
    fi # end phase DD
    # -------------------------------------------------------------------------

done

(($OPT_PHASE_C)) && inform "control files fetched:"\
 "prepared to mirror Debian $VER $DIST"

true

