# Fetch pool files.

if (($OPT_PHASE_E)); then

    ((!$OPT_SUMS)) && inform "fetching pool files..."

    # List data for actions to take later.
    for DIST1 in "$DIST" $( (($OPT_BACKPORTS)) && echo "$DIST_BACKPORTS" ); do

        # Appraise whether backports.
        IS_BACKPORTS=1; [ "$DIST1" = "$DIST" ] && IS_BACKPORTS=0

        # The following code is a bit of a spaghetti in how the various
        # filepath*.list files are made and deleted. C++ RAII would have
        # helped.

        # Extract filepaths from control files.
        #
        # If $OPT_SUMS, the following implcitly assumes that dists/ harbors no
        # irrelevant files named "Sources", "Packages" or "SHA256SUMS". The
        # assumption seems safe enough, but better software engineering would
        # probably avoid the assumption.
        touch $TMPDIR/filepaths0.list
        DIST_REALPATH="$(realpath -e "$TARGET/dists/$DIST1")"
        for CTRL in\
            $(find "$DIST_REALPATH" -name Sources)
        do
            ((!$OPT_SUMS)) && inform_of_scan "$CTRL"
            AREA0="${CTRL#$(printf '%q' "$DIST_REALPATH")/}"
            AREA="${AREA0%%/*}"
            filepaths_from_sources_ctrl "$CTRL" "$AREA"\
             >>$TMPDIR/filepaths0.list
        done
        for CTRL in\
            $(find "$DIST_REALPATH" -name Packages)
        do
            ((!$OPT_SUMS)) && inform_of_scan "$CTRL"
            filepaths_from_packages_ctrl "$CTRL" >>$TMPDIR/filepaths0.list
        done
        for CTRL in\
            $(find "$DIST_REALPATH" -name SHA256SUMS)
        do
            FP_FROM_SHA=$(mktemp)
            DIR="$(dirname "${CTRL#$(printf '%q' "$TARGET")/}")"
            filepaths_from_sha_ctrl "$CTRL" "$DIR" >>$FP_FROM_SHA
            if ((!$OPT_SUMS)); then
                inform_of_scan "$CTRL"
                inform "the last scan selects these:"
                cat $FP_FROM_SHA
                print_bar
            fi
            cat $FP_FROM_SHA >>$TMPDIR/filepaths0.list
            rm $FP_FROM_SHA
        done

        if ((!$OPT_SUMS)); then
            # Sort filepaths into locally available and otherwise.
            touch $TMPDIR/filepaths{,2}.list
            (($IS_BACKPORTS)) && touch $TMPDIR/backports-filepaths-rp.list
            for FP1 in $(<$TMPDIR/filepaths0.list); do
                # Note that locally available files outside the pool could not
                # be straightforwardly copied by this phase's method because a
                # straightforward copy would mishandle directory links (which
                # the pool fortunately lacks). This phase's method depends on
                # timestamps of files in the pool not to change, anyway,
                # whereas dists though much smaller is more complex. The
                # maintainer is less confident of timestamping practices in
                # dists. For simplicity and reliability, therefore, at the cost
                # of some strictly unnecessary data transfers over the network
                # during a successive execution of the script (but only for the
                # small dists, not for the big pool) files of dists are
                # redownloaded during each execution.
                if [[ "$FP1" = pool/* ]] && [ -f "$OLD/$FP1" ]; then
                    echo >>$TMPDIR/filepaths2.list "$FP1"
                else
                    echo >>$TMPDIR/filepaths.list "$FP1"
                    (($IS_BACKPORTS)) && [[ "$FP1" = pool/* ]]\
                     && echo >>$TMPDIR/backports-filepaths-rp.list "$FP1"
                fi
            done
        fi

        # Save the paths files.
        if (($OPT_SUMS)); then
            if (($IS_BACKPORTS)); then
                mv $VV $TMPDIR/{filepaths0,backports-filepaths}.list
                touch $TMPDIR/backports-filepaths2.list
            else
                mv $VV $TMPDIR/{filepaths0,release-filepaths}.list
                touch $TMPDIR/release-filepaths2.list
            fi
        else
            rm $VV $TMPDIR/filepaths0.list
            if (($IS_BACKPORTS)); then
                mv $VV $TMPDIR/{,backports-}filepaths.list
                mv $VV $TMPDIR/{,backports-}filepaths2.list
            else
                mv $VV $TMPDIR/{,release-}filepaths.list
                mv $VV $TMPDIR/{,release-}filepaths2.list
            fi
        fi

    done

    # At this point, the following lists exist:
    #
    #   + release-filepaths
    #   + release-filepaths2
    #   + backports-filepaths
    #   + backports-filepaths2
    #
    # These four lists are exclusive. The *-filepaths2 lists list pool files
    # that can be copied locally rather than retrieved. (If $OPT_SUMS, then
    # the *-filepaths2 are empty.) All nonpool files go to *-filepaths: no
    # local copying.
    #
    # If !$OPT_SUMS, then this list exists, too:
    #
    #   + backports-filepaths-rp
    #
    # The -rp stands for "rsyncable pool". This list is nonexclusive, but lists
    # the subset of backports-filepaths the script may try to retrieve from the
    # primary mirror before falling back to rely on snapshot. All files the
    # backports-filepaths-rp list lists are pool files.

    if (($OPT_SUMS)); then

        # Sort and print checksums.
        if (($OPT_BACKPORTS)); then
            sort_by_filename $TMPDIR/{ctrl,release,backports}-filepaths.list
        else
            sort_by_filename $TMPDIR/{ctrl,release}-filepaths.list
        fi

    else

        # Fetch remote files (and copy locally available files).

        # Copy locally available stable pool files.
        copy_locally_available $TMPDIR/release-filepaths2.list

        # Fetch stable files, including nonpool files, the primary mirror must
        # deliver. (Note that, for files already locally present, the primary
        # mirror must still refresh the local timestamps.)
        touch $TMPDIR/must-deliver.list
        cat >$TMPDIR/must-deliver.list $TMPDIR/release-filepaths{,2}.list
        [ -s $TMPDIR/must-deliver.list ]\
         && run_by_batch --persist rsync '-lKtRv --progress --stats'\
        <(
            for A in $(cat2 $TMPDIR/must-deliver.list); do
                echo "::$ARCHIVE_NAME/$A"
            done
        ) "$TARGET/" "$MIRROR_PRIMARY"

        if (($OPT_BACKPORTS)); then

            # Copy locally available backports pool files.
            copy_locally_available $TMPDIR/backports-filepaths2.list

            # Fetch backports files the primary mirror may deliver. (Unlike the
            # earlier, must-deliver files, these may-deliver files are all
            # pool files.)
            touch $TMPDIR/may-deliver.list
            cat >$TMPDIR/may-deliver.list\
             $TMPDIR/backports-filepaths-rp.list\
             $TMPDIR/backports-filepaths2.list
            [ -s $TMPDIR/may-deliver.list ]\
             && run_by_batch --persist rsync\
             '-lKtRv --ignore-missing-args --progress --stats'\
            <(
                for A in $(cat2 $TMPDIR/may-deliver.list); do
                    echo "::$ARCHIVE_NAME/$A"
                done
            ) "$TARGET/" "$MIRROR_PRIMARY"

            # From snapshot, fetch backports files, including nonpool files,
            # that are still missing. (Because wget rather than rsync is used,
            # timestamps of files already locally present are not touched.)
            touch $TMPDIR/backports-missing.list
            for A in $(<$TMPDIR/backports-filepaths.list); do
                [ -e "$TARGET/$A" ] || echo "$A"\
                >>$TMPDIR/backports-missing.list
            done
            [ -s $TMPDIR/backports-missing.list ]\
             && run_by_batch wget_insist\
             "-nc -x -nH --cut-dirs=4 -P $TARGET/pool"\
            <(
                URL0="https://$SNAPSHOT_SERVICE/$ARCHIVE_NAME/$SNAPSHOT_SSDATE"
                for A in $(cat2 $TMPDIR/backports-missing.list); do
                    echo "$URL0/$A"
                done
            ) ''
        fi

    fi

    ((!$OPT_SUMS)) && inform "pool files fetched"

fi

true

