# Install the stable archive, et al.

if (($OPT_PHASE_G)); then

    inform\
     "stamping directories,"\
     "installing the stable archive, symlinking currents, and"\
     "setting permissions..."

    # Let directory modification times equal release time.
    for DIR in $(find "$TARGET" "$ISOCD_TARGET" -type d); do
        touch -hr "$TMPDIR/release.stamp" "$DIR"
    done

    # Install the stable archive; symlink currents.
    if (($OPT_PHASE_C)); then
        [ -e "$TARGET/dists/$DIST_CODENAME" ] && die "cannot move"\
          "the stable archive to $TARGET/dists/$DIST_CODENAME,"\
          "for that node already exists"
        mv $VV "$TARGET/dists/$DIST" "$TARGET/dists/$DIST_CODENAME"
        ln_touch "$DIST_CODENAME" "$TARGET/dists/$DIST"
        ln_touch "$DIST_CODENAME" "$TARGET/dists/stable"
        (($OPT_BACKPORTS)) && ln_touch\
         "$DIST_BACKPORTS" "$TARGET/dists/stable-backports"
        touch -hr "$TMPDIR/release.stamp" "$TARGET/dists"
    fi
    if (($OPT_PHASE_F)); then
        ln_touch "$ISOCD_VER" "$ISOCD_TARGET/current"
        touch -hr "$TMPDIR/release.stamp" "$ISOCD_TARGET"
    fi

    # Verify that the mirrored archives have nothing
    # but files, directories and symlinks.
    if [[ -n "$(
        find "$TARGET" "$ISOCD_TARGET" ! -type f -a ! -type d -a ! -type l
    )" ]]; then
        die "something other than a file, directory"\
        "or symlink is for some reason found in the mirrored archive"
    fi

    # Check for broken symlinks.
    if [[ -n "$(
        find -L "$TARGET" "$ISOCD_TARGET" -type l
    )" ]]; then
        MESSAGE="broken symlink"
        if (($OPT_SMALL_TEST)); then
            inform "$MESSAGE"
        else
            die "$MESSAGE"
        fi
    fi

    # Let symlink modification times equal release time if the symlink
    # in question points (directly or indirectly) to a directory.
    for LINK in $(find "$TARGET" "$ISOCD_TARGET" -type l); do
        [ -d "$(realpath -e "$LINK")" ]\
        && touch -hr "$TMPDIR/release.stamp" "$LINK"
    done

    # Set permissions.
    for DIR  in $(find "$TARGET" "$ISOCD_TARGET" -mindepth 1 -type d); do
        chmod $VV 0775 "$DIR"
    done
    for FILE in $(find "$TARGET" "$ISOCD_TARGET" -mindepth 1 -type f); do
        chmod $VV 0664 "$FILE"
    done

    inform "done"\
     "stamping directories,"\
     "installing the stable archive, symlinking currents, and"\
     "setting permissions..."

fi

