# implementation.bash
# This is mirrorrib, by Thaddeus H. Black <thb@debian.org>
# mirrorrib: MIRROR Release Including Backports, ...
#   ... to assemble a Debian archive for local use
#
# This bash shell script implements mirrorrib.
#
# One can invoke this bash shell script directly. However, the script is
# intended as an implementational script, invoked not directly but via a
# launcher. The launcher scrubs the process environment and suppresses local
# bash configuration before launching the implementation.

set -e; set -f; shopt -s extglob
SCRIPTNAME='mirrorrib'
COMPRESS_EXT='xz|bz2|gz'
LIB_DIR="$(realpath -e "$(dirname "$0")/..")"
SCRIPT_DIR="$(realpath -e "$LIB_DIR/scripts")"
LIST_DIR="$(realpath -e "$LIB_DIR/lists")"
RC_DIR="$(realpath -e "$LIB_DIR/../../../etc")"

. $SCRIPT_DIR/03pre-initialization.bash
. $SCRIPT_DIR/05option-processing.bash
. $SCRIPT_DIR/07initialization.bash
. $SCRIPT_DIR/10function-definitions.bash
. $SCRIPT_DIR/15recursive-fetcher.bash
. $SCRIPT_DIR/20phase-a.bash  # Check that mirrors are functioning.
. $SCRIPT_DIR/30phase-b.bash  # Determine the snapshot date to use.
. $SCRIPT_DIR/40phase-cd.bash # Fetch (c) dist-root ctrl and (d) per-area ctrl.
. $SCRIPT_DIR/60phase-e.bash  # Fetch pool files.
. $SCRIPT_DIR/70phase-f.bash  # Fetch iso cd boot files.
. $SCRIPT_DIR/80phase-g.bash  # Install the stable archive, et al.
. $SCRIPT_DIR/90conclusion.bash

# IMPLEMENTATION NOTES
#
# Values of certain shell variables carry through from one script to the next.
# The program has grown a little too large for such lack of modularization to
# remain comfortable.
#
# The several files into which the program has been split have been roughly
# planned with future modularization in mind, though. As long as one does not
# unrealistically expect a collection of shell scripts like this program's to
# achieve the level of organization of (say) a C++ source, each file ought to
# make a more or less reasonable module -- if future development should find
# further modularization sufficiently desirable to justify the effort.
#
# Meanwhile, nonobvious instances of shell-variable carry-through have been
# commented to the extent to which the necessity for a comment has come to the
# maintainer's attention.

